"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
const modules_1 = require("../lib/modules");
const ssh_config_1 = require("../lib/ssh-config");
const CREATE_NEW_APP_CHOICE = 'createNewApp';
let LinkCommand = class LinkCommand extends cli_utils_1.Command {
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let [appId] = inputs;
            const { create } = options;
            if (appId && create) {
                throw this.exit(`Sorry--cannot use both ${chalk.green('app_id')} and ${chalk.green('--create')}. You must either link an existing app or create a new one.`);
            }
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let [appId] = inputs;
            let { create, name } = options;
            const config = yield this.env.config.load();
            const project = yield this.env.project.load();
            if (project.app_id) {
                const confirm = yield this.env.prompt({
                    type: 'confirm',
                    name: 'confirm',
                    message: `App ID ${chalk.green(project.app_id)} already exists in project config. Would you like to link a different app?`
                });
                if (!confirm) {
                    this.env.log.info('Not linking.');
                    return;
                }
            }
            if (!(yield this.env.session.isLoggedIn())) {
                yield cli_utils_1.promptToLogin(this.env);
            }
            if (config.backend === cli_utils_1.BACKEND_PRO) {
                const sshConfigPath = ssh_config_1.getSSHConfigPath();
                const conf = yield ssh_config_1.loadSSHConfig(sshConfigPath);
                if (!ssh_config_1.findSSHConfigHostSection(conf, config.git.host)) {
                    this.env.log.info(`SSH config settings for ${chalk.bold(config.git.host)} not found in ${chalk.bold(cli_utils_1.prettyPath(sshConfigPath))}.`);
                    this.env.log.info('Running automatic SSH setup.');
                    yield this.runcmd(['ssh', 'setup']);
                }
            }
            if (appId) {
                if (appId === project.app_id) {
                    return this.env.log.info(`Already linked with app ${chalk.bold(appId)}.`);
                }
                this.env.tasks.next(`Looking up app ${chalk.bold(appId)}`);
                const token = yield this.env.session.getAppUserToken(appId);
                const req = this.env.client.make('GET', `/apps/${appId}`)
                    .set('Authorization', `Bearer ${token}`)
                    .send();
                const res = yield this.env.client.do(req);
                if (!cli_utils_1.isAppResponse(res)) {
                    throw cli_utils_1.createFatalAPIFormat(req, res);
                }
                this.env.tasks.end();
            }
            else if (!create) {
                this.env.tasks.next(`Looking up your apps`);
                let apps = [];
                const token = yield this.env.session.getUserToken();
                const paginator = this.env.client.paginate(() => this.env.client.make('GET', '/apps').set('Authorization', `Bearer ${token}`), cli_utils_1.isAppsResponse);
                for (let r of paginator) {
                    const res = yield r;
                    apps = apps.concat(res.data);
                }
                this.env.tasks.end();
                const createAppChoice = {
                    name: 'Create a new app',
                    id: CREATE_NEW_APP_CHOICE,
                };
                const linkedApp = yield this.env.prompt({
                    type: 'list',
                    name: 'linkedApp',
                    message: `Which app would you like to link`,
                    choices: [createAppChoice, ...apps].map((app) => ({
                        name: app.id !== CREATE_NEW_APP_CHOICE ? `${app.name} (${app.id})` : chalk.bold(app.name),
                        value: app.id
                    }))
                });
                appId = linkedApp;
            }
            if (create || appId === CREATE_NEW_APP_CHOICE) {
                const token = yield this.env.session.getUserToken();
                if (config.backend === cli_utils_1.BACKEND_PRO) {
                    if (!name) {
                        name = yield this.env.prompt({
                            type: 'input',
                            name: 'name',
                            message: 'Please enter a name for your new app:',
                        });
                    }
                    const req = this.env.client.make('POST', '/apps')
                        .set('Authorization', `Bearer ${token}`)
                        .send({ name });
                    const res = yield this.env.client.do(req);
                    if (!cli_utils_1.isAppResponse(res)) {
                        throw cli_utils_1.createFatalAPIFormat(req, res);
                    }
                    appId = res.data.id;
                    yield this.runcmd(['config', 'set', 'app_id', appId]);
                    yield this.runcmd(['git', 'remote']);
                    this.env.log.ok(`Project linked with app ${chalk.bold(appId)}!`);
                }
                else {
                    const opn = modules_1.load('opn');
                    opn(`${config.urls.dash}/?user_token=${token}`, { wait: false });
                    this.env.log.info(`Rerun ${chalk.green(`ionic link`)} to link to the new app.`);
                }
            }
            else {
                yield this.runcmd(['config', 'set', 'app_id', appId]);
                if (config.backend === cli_utils_1.BACKEND_PRO) {
                    yield this.runcmd(['git', 'remote']);
                }
                this.env.log.ok(`Project linked with app ${chalk.bold(appId)}!`);
            }
        });
    }
};
LinkCommand = tslib_1.__decorate([
    cli_utils_1.CommandMetadata({
        name: 'link',
        type: 'project',
        backends: [cli_utils_1.BACKEND_LEGACY, cli_utils_1.BACKEND_PRO],
        description: 'Connect your local app to Ionic',
        longDescription: `
If you have an app on Ionic, you can link it to this local Ionic project with this command.

Excluding the ${chalk.green('app_id')} argument looks up your apps on Ionic and prompts you to select one.

This command simply sets the ${chalk.bold('app_id')} property in ${chalk.bold(cli_utils_1.PROJECT_FILE)} for other commands to read.
  `,
        exampleCommands: ['', 'a1b2c3d4', '--create --name "My New App"'],
        inputs: [
            {
                name: 'app_id',
                description: `The ID of the app to link (e.g. ${chalk.green('a1b2c3d4')})`,
                required: false,
            },
        ],
        options: [
            {
                name: 'create',
                description: 'Create a new app on Ionic and link it with this local Ionic project',
                backends: [cli_utils_1.BACKEND_PRO],
                type: Boolean,
            },
            {
                name: 'name',
                description: 'The app name to use during the linking of a new app',
            },
        ],
    })
], LinkCommand);
exports.LinkCommand = LinkCommand;
